// Elevator Video Switcher
// Written by Ben Peoples of Trinculo's Attic, April 23-24, 2012.
// This is released as beerware: You are free to use and modify 
// this, if you find it useful, please buy me a beer.
// (c) 2012 Trinculo's Attic.
// Contact: ben@trinculosattic.com


int delta = 0; // This stores the current difference between the high and the low.

void setup()
{
  pinMode(2,OUTPUT);
  pinMode(3,OUTPUT);
  pinMode(4,OUTPUT);
  pinMode(5,OUTPUT);
  digitalWrite(12,HIGH);
  digitalWrite(11,HIGH);

  Serial.begin(9600);

  // Commented out here let us use a Serial LCD for debugging  
  //Serial.write(0xFE);
  //Serial.write(0x01);
  //Serial.write(0x7C);
  //Serial.write(157);
}

// "Zero" is the position for the first floor
int zero;
int zero_accum;
uint8_t in_position0 = 0;
uint8_t was_down = 0;
uint8_t triggered0 = 0;
uint8_t triggered0_alt = 0;

// "One" is the position for the second floor
int one;
int one_accum;
uint8_t in_position1 = 0;
uint8_t was_up = 0;
uint8_t triggered1 = 0;
uint8_t triggered1_alt = 0;

void loop()
{
  /* // Commented out code has to do with LCD debugging info
   //    Serial.write(0xFE);
   //Serial.write(0x80);
   Serial.print(millis());
   Serial.print(',');
   Serial.print(zero);
   //Serial.write(0xFE);
   //Serial.write(0x80+64);
   Serial.print(',');
   Serial.println(one);
   */
  // Read sensors
  zero = analogRead(4);
  one = analogRead(5);

  // Take delta of them
  delta = zero - one;

  // This is an accumulator "debouncer" -- it lets us require a
  // largish number of sequential reads to hit a state.
  // In short, one false (or true) read doesn't equal a state change, but 20 does.

  // The values for delta were determined experimentally. 

  if(delta > 40 && zero_accum < 20)
  {
    zero_accum++;
  }
  else if(zero_accum > 0)
  {
    zero_accum--;
  }

  if(delta < -100 && one_accum < 20)
  {
    one_accum++;
  }
  else if(one_accum > 0)
  {
    one_accum--; 
  }


  // If we've accumulated enough to establish that we're at the first floor, then do stuff.
  if(zero_accum == 20)
  {
    triggered0 = 0;
    in_position0 = 1;
    was_down = 1; // First floor is also called "down"
    was_up = 0;
  }  

  // We've moved off the first floor 
  if(zero_accum == 0)
  {
    in_position0 = 0; // So we are no longer in position 0.
  }  

  // If we've accumulated enough to establish that we're at the second floor, then do stuff.
  if(one_accum == 20)
  {
    triggered1 = 0;

    in_position1 = 1;
    was_up = 1;
    was_down = 0; 
  }

  // We've moved off the second floor
  if(one_accum == 0)
  {
    in_position1 = 0; // So we are no longer in position 1.
  }  

  // If we are at the first (zero) floor, and we have not triggered the alternate, then
  if(in_position0 == 1 && triggered0_alt == 0)
  {
    delay(5000); // Delay 5 seconds
    triggered0_alt = 1; // Change state to indicate that we've triggered (so we don't trigger again)
    digitalWrite(4,HIGH); // Press key 3.
    delay(100); // For 1/10th of a second
    digitalWrite(4,LOW); // Unpress key 3.
  }

  // If we are at the second (one) floor, and we have no triggered the alternate, then
  if(in_position1 == 1 && triggered1_alt == 0)
  {
    triggered1_alt = 1; // Change state to indicate we've triggered
    delay(5000); // Wait 5 seconds
    digitalWrite(5,HIGH); // Press key 4.
    delay(100); // For 1/10th of a second.
    digitalWrite(5,LOW); // Unpress key 4.
  }

  // If we were at the first (down) floor, but we've moved off position 0, and we haven't yet triggered
  if(was_down == 1 && in_position0 == 0 && triggered0 == 0)
  {
    triggered0_alt = 0; // Reset the alternate trigger
    triggered0 = 1; // Toggle our trigger to say we've done it.
    Serial.println("Triggered Going Up"); // Print out something useful for debugging
    digitalWrite(2,HIGH); // Press key 1.
    delay(100); // For 1/10th of a second
    digitalWrite(2,LOW);  // Unpress key 1.
  }

  // If we were at the second (up) floor, but we've moved off position 1, and we haven't yet triggered
  if(was_up == 1 && in_position1 == 0 && triggered1 == 0)
  {
    triggered1_alt = 0; // Reset the alternate trigger
    triggered1 = 1; // Toggle our trigger to say we've done it
    Serial.println("Triggered Going Down"); // Print out useful debugging info
    digitalWrite(3,HIGH); // Press key 2
    delay(100); // For 1/10th of a second
    digitalWrite(3,LOW); // Unpress key 2.
  }

}

